"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportSarif = exports.main = exports.relative = exports.minVal = void 0;
var fs = require('fs');
var _a = require('node-sarif-builder'), SarifBuilder = _a.SarifBuilder, SarifRunBuilder = _a.SarifRunBuilder, SarifResultBuilder = _a.SarifResultBuilder;
var yargs = require('yargs');
function minVal(val) {
    if (val) {
        return val;
    }
    return 1;
}
exports.minVal = minVal;
function relative(rootdir, fullpath) {
    if (fullpath.startsWith(rootdir)) {
        return fullpath.substring(rootdir.length + 1);
    }
    return fullpath;
}
exports.relative = relative;
function main() {
    // TODO add a relative directory option
    var argv = yargs
        .option('o', {
        alias: 'output',
        describe: 'Output filename',
        type: 'string',
        demandOption: false,
    })
        .option('r', {
        alias: 'root',
        describe: 'Root directory',
        type: 'string',
        demandOption: false,
    })
        .demandCommand(1)
        .usage('Usage: $0 [options] <filename>').argv;
    var filename = argv._[0];
    var outputfilename = argv.output;
    var rootdir = argv.root;
    exportSarif(filename, outputfilename, rootdir);
}
exports.main = main;
function exportSarif(filename, outputfilename, rootdir) {
    var results = JSON.parse(fs.readFileSync(filename, 'utf8'));
    // SARIF builder
    var sarifBuilder = new SarifBuilder();
    // SARIF Run builder
    var sarifRunBuilder = new SarifRunBuilder().initSimple({
        toolDriverName: 'spectral-sarif',
        toolDriverVersion: '0.0.8',
    });
    results.forEach(function (result) {
        var sarifResultBuilder = new SarifResultBuilder();
        var sarifResultInit = {
            ruleId: result.code,
            level: 'error',
            messageText: result.message,
            // TODO needs to be relative to the sonar source directory
            fileUri: relative(rootdir, result.source),
            startLine: 0,
            startColumn: 0,
            endLine: 0,
            endColumn: 0
        };
        sarifResultInit.startLine = minVal(result.range.start.line) + 1;
        sarifResultInit.startColumn = minVal(result.range.start.character);
        sarifResultInit.endLine = minVal(result.range.end.line) + 1;
        sarifResultInit.endColumn = minVal(result.range.end.character);
        sarifResultBuilder.initSimple(sarifResultInit);
        sarifRunBuilder.addResult(sarifResultBuilder);
    });
    sarifBuilder.addRun(sarifRunBuilder);
    var json = sarifBuilder.buildSarifJsonString({ indent: true });
    if (outputfilename) {
        fs.writeFileSync(outputfilename, json);
    }
    else {
        console.log(json);
    }
}
exports.exportSarif = exportSarif;
